/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: NewAppWizard.java,v 1.1.2.10 2000/02/13 09:50:27 pmahar Exp $
 */

package org.enhydra.devsys.wizard.newapp;

import org.enhydra.devsys.swing.SwingUtil;
import org.enhydra.devsys.wizard.common.WizardDialog;
import org.enhydra.devsys.wizard.common.WizardPage;
import java.io.*;
import java.util.Properties;
import java.awt.Component;
import java.awt.Dialog;
import java.awt.Frame;
import javax.swing.ImageIcon;
import javax.swing.JOptionPane;
import com.lutris.devsys.newApp.LutrisAppTree;
import com.lutris.devsys.newApp.NewAppException;

/**
 * The NewAppWizard provides a wizard interface to the LutrisAppTree command
 * line new application utility.
 *
 * @author
 * Paul Mahar
 * @version %I%, %G%
 */
public class NewAppWizard implements NewAppProperties {
    private final String SERVER_RUN_CLASS =
	"com.lutris.servlet.multiServer.MultiServer";
    private final String TITLE = "Enhydra Application Wizard";
    private final String INSTRUCTIONS =
	"Fill in the following fields to quickly define and "
	+ "create a new Enhydra XMLC application. "
	+ "The application will consist of an HTML "
	+ "file, a presentaion class and an Enhydra application class. ";
   private final String FINISH_DIALOG_MESSAGE =
	  "The application has been generated. You can now "
        + "compile the HTML file with XMLC to generate "
	+ "a java class containing a DOM (document object model). "
	+ "The presentaion class works with the DOM to "
	+ "generate dynamic HTML. MultiServer uses the Enhydra "
	+ "application class to launch the XMLC application.";
    private final String PAGE_TITLE =
	"Enter the Enhydra application details.";
    private final String UNIX_ENHYDRA_DIR = "/usr/local/enhydra";
    private final String WIN32_ENHYDRA_DIR = "C:\\usr\\local\\enhydra";
    private String       enhydraDir = UNIX_ENHYDRA_DIR;    // default
    private String       appName = "untitled";		   // default
    private String       rootDir = "/usr/local";	   // default;
    private String       packageName = "untitled";	   // default
    private int		 choice = WizardDialog.CHOICE_CANCEL;
    private NewAppPage   page;

    /**
     * Create a new application wizard
     */
    public NewAppWizard() {
	initEnhydraDir();
    }

    /**
     * Get the full path to the application directory. This combines
     * the root directory and the application name.
     */
    public String getAppDir() {
	String appDir = getRootDir();

	if (!appDir.endsWith(File.separator)) {
	    appDir += File.separator;
	}

	appDir += getAppName();

	return appDir;
    }

    /**
     * Get the Enhydra home directory. The default is /usr/local/enhydra.
     * <P><I>
     * NewAppProperties implementation
     * </I></P>
     */
    public String getEnhydraDir() {
	return enhydraDir;
    }

    /**
     * Set the Enhydra home directory. The default is /usr/local/enhydra.
     * <P><I>
     * NewAppProperties implementation
     * </I></P>
     */
    public void setEnhydraDir(String d) {
	enhydraDir = d;
        Properties prop = new Properties();
        String     fn = System.getProperties().getProperty("user.home")
			    + File.separator + "enhydra.properties";
	File       f = new File(fn);
        try {
           if (f.exists()) {
	     FileInputStream i = new FileInputStream(f);
   	     FileOutputStream o = new FileOutputStream(f);
   	     prop.load(i);
             prop.put("EnhydraDir", enhydraDir);  // use setProperty() in Java 2
             prop.save(o, "Enhydra Properties");  // use store() in Java 2
           } else {
	     prop.put("EnhydraDir", enhydraDir);  // use setProperty() in Java 2
   	     FileOutputStream o = new FileOutputStream(f);
             prop.save(o, "Enhydra Properties");  // use store() in Java 2
          }
	} catch (java.io.FileNotFoundException e) {
	    e.printStackTrace();
	} catch (java.io.IOException e) {
	    e.printStackTrace();
	}

    }

    /**
     * Get the name to use when generating an Enhydra app.
     * <P><I>
     * NewAppProperties implementation
     * </I></P>
     */
    public String getAppName() {
	return appName;
    }

    /**
     * Set the name to use when generating an application.
     * <P><I>
     * NewAppProperties implementation
     * </I></P>
     */
    public void setAppName(String n) {
	appName = n.trim().toLowerCase();
    }

    /**
     * Get the root directory where to create the application directory.
     * <P><I>
     * NewAppProperties implementation
     * </I></P>
     */
    public String getRootDir() {
	return rootDir;
    }

    /**
     * Set the root directory where to create the application directory.
     * <P><I>
     * NewAppProperties implementation
     * </I></P>
     */
    public void setRootDir(String d) {
	rootDir = d;
    }

    /**
     * Get the package name to use when generating an application.
     * <P><I>
     * NewAppProperties implementation
     * </I></P>
     */
    public String getPackageName() {
	return packageName;
    }

    /**
     * Set the package name to use when generating an application. Only
     * a single package is supported.
     * <P><I>
     * NewAppProperties implementation
     * </I></P>
     */
    public void setPackageName(String n) {
        packageName = n.toLowerCase().trim();
    }

    /**
     * Open the wizard as a child of the specified owner.
     *
     * @param owner
     * A frame or dialog that acts as the parent of the modal wizard dialog.
     *
     * @return
     * Returns the last page selected by the user.
     */
    public WizardPage invokeWizard(Component owner) {
	page = new NewAppPage();

	page.setPageTitle(getPageTitle());
	page.setInstructions(getInstructions());
	writeAppProperties(page);
	showDialog(owner);
	readAppProperties(page);

	if (choice == WizardDialog.CHOICE_OK) {
	    createApp(owner);
	}

	return page;
    }

    /**
     * Get the instructions to show on all pages related to this wizard.
     */
    public final String getInstructions() {
	return INSTRUCTIONS;
    }

    /**
     * Get the page title to show on all pages related to this wizard.
     */
    public final String getPageTitle() {
	return PAGE_TITLE;
    }

    /**
     * Get the title to show in the caption bar of this wizard.
     */
    public final String getTitle() {
	return TITLE;
    }

    /**
     * Get the class to run to start MultiServer.
     */
    public final String getServerRunClass() {
	return SERVER_RUN_CLASS;
    }

    /**
     * Open the wizard dialog using the specified owner as the parent.
     *
     * @param owner
     * The owner is the parent of the modal wizard dialog.
     */
    public void showDialog(Component owner) {
	WizardDialog dialog = null;

	if (owner instanceof Dialog) {
	    dialog = new WizardDialog((Dialog) owner, getTitle(), true);
	} else if (owner instanceof Frame) {
	    dialog = new WizardDialog((Frame) owner, getTitle(), true);
	}

	dialog.addWizardPage(page);
	dialog.show();

	choice = dialog.getChoice();

	dialog.clearAll();

	if (owner.isVisible()) {
	    owner.requestFocus();
	}
    }

    public int getChoice() {
      return choice;
    }

    /**
     * Create the Enhydra application directories structure and sample
     * welcome classes.
     *
     * @param owner
     * The owner is the parent to use when displaying the modal error dialogs.
     */
    public void createApp(Component owner) {
	boolean continueBuild = true;

	try {
	    createAppTree();
	} catch (NewAppException e) {
	    continueBuild = false;

	    JOptionPane.showMessageDialog(owner, e.getMessage(),
					  "Error generating application",
					  JOptionPane.ERROR_MESSAGE);
	}

	if (continueBuild) {
	    createAppFiles();
	}
    }

    /**
     * Create a readme HTML with a link for viewing the generated
     * sample welcome page.
     *
     *
     * @param outFile
     * File to write html file into.
     *
     * @throws IOException
     *
     */
    public void writeReadme(String[] steps) throws IOException {
        File outFile = getReadme();
        if (! outFile.exists()) {
  	  PrintWriter out = new PrintWriter(new FileWriter(outFile));

	  // head
	  out.println("<HTML>");
	  out.println("<HEAD>");
	  out.println("<TITLE>Readme for your new XMLC Enhydra application</TITLE>");
	  out.println("</HEAD>");

	  // Body
	  out.println("<BODY>");
	  out.println();
          out.println("<H2>To run your sample application</H2>");
          out.println("<OL>");
          for (int i = 0; i < steps.length ; i++) {
  	    out.println("<LI>" + steps[i]);
          }
          out.println("</OL>");
          out.println("<P>");
          out.println("If the already server is running on port 9000, you ");
          out.println("may view the Welcome page by clicking on the ");
          out.println("following link.");
          out.println("</P>");
	  out.println("<A HREF=http://localhost:9000>http://localhost:9000</A>");
	  out.println();
	  out.println("</BODY>");
	  out.println("</HTML>");
	  out.println();
	  out.close();
        }
    }

    public File getStartMultiServer() {
    	String javaFile = getPackageDir() + File.separator
	                 + "StartMultiServer.java";
        return new File(javaFile);
    }

    /**
     * Write a StartMultisServer.java file for launching and stopping MultiServer
     * with a simple graphical interface. This is useful in environments such as
     * the Win32 version of JBuilder 3 where you cannot run an arbitrary class
     * from a project.
     *
     * @throws IOException
     */
    public void writeStartMultiServer() throws IOException {
       if (! getStartMultiServer().exists()) {
	PrintWriter out = new PrintWriter(new FileWriter(getStartMultiServer()));

        if (getPackageName() != null && getPackageName().trim().length() > 0) {
   	   out.println("package " + getPackageName() + ";");
         }
	out.println("");
	out.println("import javax.swing.*;");
	out.println("import java.awt.*;");
	out.println("import java.awt.event.*;");
	out.println("");
	out.println("public class StartMultiServer extends JFrame {");
	out.println("  JButton button = new JButton();");
	out.println("");
	out.println("  public static void main(String[] args) {");
	out.println("    if (args == null || args.length < 1) {");
	out.println("      args = new String[1];");
        out.println("      args[0] = \"" +
            doubleEscape(getRootDir() + "\\" + getAppName() + "\\output\\multiserver.conf")
              + "\";");
	out.println("     }");
	out.println("    com.lutris.servlet.multiServer.MultiServer.main(args);");
	out.println("    StartMultiServer frame = new StartMultiServer();");
	out.println("    frame.show();");
	out.println("  }");
	out.println("");
	out.println("    public StartMultiServer() {");
	out.println("      button.setText(\"Stop\");");
	out.println("      button.addActionListener(new ActionListener() {");
	out.println("        public void actionPerformed(ActionEvent e) {");
	out.println("         System.exit(0);");
	out.println("        }");
	out.println("      });");
	out.println("      getContentPane().add(button);");
	out.println("      enableEvents(java.awt.AWTEvent.WINDOW_EVENT_MASK);");
	out.println("      setTitle(\"MultiServer\");");
	out.println("      setSize(200,100);");
	out.println("      setLocation(new Point(200,200));");
	out.println("    }");
	out.println("");
	out.println("  protected void processWindowEvent(java.awt.event.WindowEvent e) {");
	out.println("    super.processWindowEvent(e);");
	out.println("    if (e.getID() == java.awt.event.WindowEvent.WINDOW_CLOSING) {");
	out.println("      System.exit(0);");
	out.println("    }");
	out.println("  }");
	out.println("");
	out.println("}");
	out.println("");
	out.flush();
	out.close();
       }
    }

    private String doubleEscape(String in) {
      StringBuffer doubled = new StringBuffer();
      int index = in.indexOf("\\");
      while (index > -1) {
        doubled.append(in.substring(0,index));
        doubled.append("\\\\");
        if (index == in.lastIndexOf("\\")) {
          if (! in.endsWith("\\")) {
            doubled.append(in.substring(index + 1));
          }
          in = "";
        } else {
          in = in.substring(index + 1);
        }
        index = in.indexOf("\\");
      }
      return doubled.toString();
    }

    /**
     * Create application files not generated by the LutrisAppTree class and
     * modify files to work with current Enhydra home directory.
     */
    public void createAppFiles() {
	File f;

	createDirectories();

	try {

	    // create log file
	    f = getLogFile();

	    if (!f.exists()) {
		FileOutputStream o = new FileOutputStream(f);
                o.write(32); // use f.createNewFile() in Java 2
	    }

	    // do make's job of fixing multiserver.conf.in
	    // otherwise you can't run in JBuilder until you run make!
	    File outFile = getMultiServerConf();

	    if (!outFile.exists()) {

		// make runs sed on multiserver.conf.in to replace %OUTPUT% with
		// the output directory.
		String multiServerConfIn = getPackageDir() + File.separator
					   + "multiserver.conf.in";

		f = new File(multiServerConfIn);

		if (f.exists()) {
		    BufferedReader r = null;
		    BufferedWriter w = null;

		    try {
			r = new BufferedReader(new FileReader(f));
			w = new BufferedWriter(new FileWriter(outFile));

			String outDirString = getOutDir();

			/*
			 * String outDirString = "";
			 * if (outDir.charAt(1) == ':') {
			 * //              outDirString = "//" + outDir.charAt(0) + outDir.substring(2);
			 * outDirString = outDir.substring(2);
			 * }
			 * if (File.separator.equals("\\")) {
			 * outDirString = outDirString.replace('\\', '/');
			 * }
			 */
			String s;

			while ((s = r.readLine()) != null) {
			    int i = s.indexOf("@OUTPUT@");

			    if (i != -1) {

				// System.out.println("found output line=" + s);
				String newline =
				    s.substring(0, i) + outDirString
				    + s.substring(i + "@OUTPUT@".length());

				// System.out.println("new line=" + newline);
				s = newline;
			    }

			    w.write(s);
			    w.newLine();
			}

			w.close();
			r.close();

			// raw copy of start script, <appname>.conf to output dir
			String appConf = getAppName() + ".conf";
			;
			String appConfIn = getPackageDir() + File.separator
					   + appConf;

			f = new File(appConfIn);

			if (f.exists()) {
			    outFile = getAppConf();

			    if (!outFile.exists()) {
				r = new BufferedReader(new FileReader(f));
				w = new BufferedWriter(new FileWriter(outFile));

				while ((s = r.readLine()) != null) {
				    w.write(s);
				    w.newLine();
				}

				w.close();
				r.close();
			    }
			}
		    } catch (Exception e) {
			e.printStackTrace();

			// DWIT- dialog to tell user he needs to run make in bash !
		    }
		    finally {
			if (w != null) {
			    w.close();
			}

			if (r != null) {
			    r.close();
			}
		    }
		}
            }
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

    /**
     * Use the LutrisAppTree class to generate the directory
     * and file structure for a simple Enhyra application.
     *
     * @throws com.lutris.devsys.newApp.NewAppException
     */
    public void createAppTree()
	    throws com.lutris.devsys.newApp.NewAppException {
	LutrisAppTree lat = new LutrisAppTree();

	lat.setAppName(getAppName());
	lat.setPackageDir(getPackageName());
	lat.setRootDir(getRootDir());
	lat.setEnhydraDir(getEnhydraDir());
	lat.create();
    }

    /**
     * Get the full path to the base directory where to generate java
     * source files. This combines the root directory and the application
     * name and the package name to form the full directory path.
     */
    public String getPackageDir() {
	String packageDir;

	packageDir = getAppDir() + File.separator + getPackageName();

	return packageDir;
    }

    /**
     * Get the output directory. The fullpath to this directory is formed
     * by combining the root directory, the application name and "output".
     */
    public String getOutDir() {
	String outDir;

	outDir = getAppDir() + File.separator + "output";

	return outDir;
    }

    public File getAppConf() {
      File f;
      String name = getOutDir() + File.separator
             +    getAppName() + ".conf";
      f = new File(name);
      return f;
    }

    /**
     * Get the full path to the multiserver.conf file that resides
     * in the output directory.
     */
    public File getMultiServerConf() {
	String conf;

	conf = getOutDir() + File.separator + "multiserver.conf";

	return new File(conf);
    }

    /**
     * Get the fullpath to the log file that resides in the output
     * directory. The log file has the same name as the application with
     * an extension of "log".
     */
    public File getLogFile() {
	File f;

	f = new File(getOutDir() + File.separator + getAppName() + ".log");

	return f;
    }

    /**
     * Get the full path to the readme file. This file resides in the
     * application directory.
     */
    public File getReadme() {
	File f;

	f = new File(getAppDir() + File.separator + "Readme.html");

	return f;
    }

    /**
     * Write the app properties to an object that implements the NewAppProperties
     * interface.
     */
    public void writeAppProperties(NewAppProperties p) {
	p.setAppName(getAppName());
	p.setPackageName(getPackageName());
	p.setRootDir(getRootDir());
	p.setEnhydraDir(getEnhydraDir());
    }

    /**
     * Read the app properties from an object that implements the NewAppProperties
     * interface.
     */
    public void readAppProperties(NewAppProperties p) {
	setAppName(p.getAppName());
	setPackageName(p.getPackageName());
	setRootDir(p.getRootDir());
	setEnhydraDir(p.getEnhydraDir());
    }

    /**
     * Run the new application wizard in standalone mode. The wizard can
     * also be invoked within an IDE.
     *
     * @param args
     * Not used.
     */
    public static void main(String[] args) {
	SwingUtil.setLookAndFeelToSystem();

	NewAppWizard wiz = new NewAppWizard();

	wiz.invokeWizard(new Frame());
	System.out.println("Done");
	System.exit(0);
    }

    /**
     * Initialize the Enhydra directory property. See if property is set in
     * the enhydra.prperties file. If it is not, check common locations for
     * Enhydra files.
     */
    private void initEnhydraDir() {
	try {
	    Properties prop = new Properties();
	    String     fn = System.getProperties().getProperty("user.home")
			    + File.separator + "enhydra.properties";
	    File       f = new File(fn);

	    if (f.exists()) {
		FileInputStream i = new FileInputStream(f);

		prop.load(i);

		enhydraDir = prop.getProperty("EnhydraDir");
	    } else {
		prop.put("EnhydraDir", enhydraDir);  // use setProperty() in Java 2

		FileOutputStream o = new FileOutputStream(f);

		prop.save(o, "Enhydra Properties");  // use store() in Java 2
	    }
	} catch (Exception e) {
	    e.printStackTrace();
	}

	if (!isEnhydraDirValid()) {
	    enhydraDir = WIN32_ENHYDRA_DIR;
	}
	if (!isEnhydraDirValid()) {
	    enhydraDir = WIN32_ENHYDRA_DIR + "2.3";
	}

	if (!isEnhydraDirValid()) {
	    enhydraDir = UNIX_ENHYDRA_DIR;
	}

	if (!isEnhydraDirValid()) {
	    enhydraDir = UNIX_ENHYDRA_DIR + "2.3";
	}

    }

    /**
     * Return true if the current enhydraDir is set to a location
     * that contains Enhydra files.
     */
    private boolean isEnhydraDirValid() {
	File f = new File(enhydraDir + File.separator + "lib"
			  + File.separator + "enhydra.jar");

	return f.exists();
    }

    /**
     * Create directories as needed for the new application.
     */
    private void createDirectories() {
	File dir;

	dir = new File(getAppDir());

	if (!dir.exists()) {
	    dir.mkdir();
	}

	dir = new File(getPackageDir());

	if (!dir.exists()) {
	    dir.mkdir();
	}

	dir = new File(getOutDir());

	if (!dir.exists()) {
	    dir.mkdir();
	}
    }

    public synchronized void validateProperties() throws PropertyException {
      int index = -1;

      // validate enhydra dir
      if (! isEnhydraDirValid()) {
        throw new PropertyException(PropertyException.BAD_ENHYDRA_DIR);
      }

      // validate packate name
      index = getPackageName().indexOf(".") +
              getPackageName().indexOf(" ");
      if (index > -1 || getPackageName().length() == 0) {
       throw new PropertyException(PropertyException.BAD_PACKAGE_NAME);
      }
      for (int i = 0 ; i < getPackageName().length() ;  i++ ) {
        if (i == 0) {
          if (! Character.isJavaIdentifierStart(getPackageName().charAt(i))) {
             throw new PropertyException(PropertyException.BAD_PACKAGE_NAME);
          }
        } else {
          if (! Character.isJavaIdentifierPart(getPackageName().charAt(i))) {
             throw new PropertyException(PropertyException.BAD_PACKAGE_NAME);
          }
        }
      }

      // validate app name
      for (int i = 0 ; i < getAppName().length() ;  i++ ) {
        if (! Character.isLetterOrDigit(getAppName().charAt(i))) {
           throw new PropertyException(PropertyException.BAD_APP_NAME);
        }
      }
    }

}



/*--- formatting done in "Sun Java Convention" style on 01-16-2000 ---*/

