/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: CompileThread.java,v 1.1.2.8 2000/02/13 03:41:11 pmahar Exp $
 */

package org.enhydra.ide.jbuilder.common.wizard.xmlc;

import org.enhydra.ide.jbuilder.common.node.OtterProject;
import org.enhydra.ide.jbuilder.common.node.OtterXMLCNode;
import org.enhydra.ide.jbuilder.common.capture.Writer;
import org.enhydra.ide.jbuilder.common.capture.WriteListener;
import org.enhydra.xml.xmlc.XMLCException;

import java.io.PrintWriter;
import java.io.File;

/**
 * CompileThread calls XMLC once for each selected HTML file.
 *
 *
 * @author
 * Paul Mahar
 * @version %I%, %G%
 */
public class CompileThread extends Thread {
    private boolean       printProgress = true;
    private boolean       fresh = true;
    private int		  rCount = 0;
    private OtterProject  project = null;
    private OtterXMLCNode[]  nodes = null;
    private PrintWriter      traceWriter = null;

    /**
     * Create a CompileThread with listeners ready to
     * recieve output from XMLC.
     *
     *
     * @param listeners
     * A listener implements an onWrite method that is called
     * whenever XMLC writes output. The writes are captured
     * for display in a wizard.
     */
    public CompileThread(WriteListener[] listeners) {
      this(listeners, false);
    }

    public CompileThread(WriteListener[] listeners,
                         boolean buffer) {
	Writer capture = new Writer();
        capture.setLineBuffered(buffer);

	for (int i = 0; i < listeners.length; i++) {
	    capture.addWriteListener(listeners[i]);
	}

	traceWriter = new PrintWriter(capture, false);
    }

    /**
     * Get the HTML source files that are to be compiled.
     */
    public OtterXMLCNode[] getNodes() {
	return nodes;
    }

    /**
     * Set the HTML source files to compile.
     */
    public void setNodes(OtterXMLCNode[] n) {
	nodes = n;
    }

    /**
     * Get the IDE project that contains the HTML source files to compile.
     */
    public OtterProject getProject() {
	return project;
    }

    /**
     * Set the IDE project that contains the HTML source files to compile.
     */
    public void setProject(OtterProject bp) {
	project = bp;
    }

    public boolean isPrintProgress() {
      return printProgress;
    }

    public void setPrintProgress(boolean b) {
      printProgress = b;
    }



    /**
     * Run the XMLC for all selected HTML files.
     */
    public void run() {
	fresh = true;

	compileSelections();
    }

    public synchronized void runInCurrentThread() {
	fresh = true;

	compileSelections();
    }

    /**
     * Make the thread stale so it stops running gracefully.
     */
    public void expire() {
	fresh = false;
    }

    /**
     * Loop through all selected HTML files and call XMLC for each
     * and write out exceptions to any registered listeners.
     */
    private void compileSelections() {
	int	   count = getNodes().length;
	XMLCRunner runner = new XMLCRunner();

	runner.setTraceWriter(traceWriter);

	for (int i = 0; i < count; i++) {
	    if (!fresh) {
		break;
	    }

	    try {
		if (getNodes()[i].isCompileSelected() && fresh) {
                    File sourceDoc = getNodes()[i].getOptions().getSourceDocument();
                    if (sourceDoc.exists()){
                       if (isPrintProgress()) {
     		          traceWriter.println("Compiling: ");
		          traceWriter.println("  "
                            + sourceDoc.getAbsolutePath());
                       }
                       getNodes()[i].preCompile();
  		       runner.setNode(getNodes()[i]);
		       runner.compile();
                    } else {
                      if (isPrintProgress()) {
                        traceWriter.println("Source document not found: " +
                          sourceDoc.getAbsolutePath());
                      }
                    }
		}
	    } catch (XMLCException e) {
                if (isPrintProgress()) {
  		  traceWriter.println("");
                }
		getNodes()[i].setException(e);

	    } catch (org.xml.sax.SAXException e) {
                if (isPrintProgress()) {
  		  traceWriter.println("");
                 }
		getNodes()[i].setException(e);
	    } catch (java.io.IOException e) {
		e.printStackTrace();
                if (isPrintProgress()) {
  		  traceWriter.println("");
                }
	    }

	    rCount++;
	}

	if (fresh) {
          if (isPrintProgress()) {
            printResults();
            printExceptions();
            traceWriter.println("");
	    traceWriter.println("--- XMLC finished compiling ---");
	    traceWriter.println("");
          }
	} else {
            printExceptions();
	    traceWriter.println("");
	    traceWriter.println("--- Compilation interrupted ---");
     	    traceWriter.println("");
	}

    }

    /**
     * Determine what files were created and write out the
     * compilation results to any registered listeners.
     */
    private void printResults() {
	int eCount = 0;

	traceWriter.println("");
	traceWriter.println("Gathering results...");

	for (int i = 0; i < rCount; i++) {
	    if (getNodes()[i].getException() != null) {
		eCount++;
	    }
	}

	sleepInSeconds(2);

	if ((eCount < rCount) && getProject().isKeepJavaSelected()) {
	    traceWriter.println("");
	    traceWriter.println("New java files:");

	    for (int i = 0; i < rCount; i++) {
		sleepInSeconds(0.75);

		if (!fresh) {
		    break;
		}

		if (getNodes()[i].isCompileSelected()
			&& getNodes()[i].isNewJavaFound()) {
		    getNodes()[i].addJavaSourceAsNode();
		    traceWriter.println(" "
					+ getNodes()[i].getOptions().getJavaClassSource());
		}
	    }
	}

	sleepInSeconds(2);

	if ((eCount < rCount) && getProject().isCompileJavaSelected()) {
	    traceWriter.println("");
	    traceWriter.println("New class files:");

	    for (int i = 0; i < rCount; i++) {
		sleepInSeconds(0.75);

		if (!fresh) {
		    break;
		}

		if (getNodes()[i].isCompileSelected()
			&& getNodes()[i].isNewClassFound()) {
		    traceWriter.println(" " + getNodes()[i].getClassFile());
		}
	    }
	}
      }

      private void printExceptions() {
      	int eCount = 0;
      	for (int i = 0; i < rCount; i++) {
	    if (getNodes()[i].getException() != null) {
		eCount++;
	    }
	}
	if (eCount > 0) {
	    traceWriter.println("");
	    traceWriter.println("Exception encountered:");

	    for (int i = 0; i < rCount; i++) {
		if (getNodes()[i].getException() != null) {
		    traceWriter.print("    ");
		    traceWriter.print(getNodes()[i].getOptions().getSourceDocument());
		    traceWriter.println(":");
		    traceWriter.println("    "
					+ getNodes()[i].getException().getMessage());
		}
	    }
	}
    }

    /**
     * Sleep the thread for a given number of seconds.
     *
     *
     * @param seconds
     * How many seconds to sleep.
     */
    private void sleepInSeconds(double seconds) {
	try {
	    Thread.sleep((int) (seconds * 1000));
	} catch (InterruptedException e) {}
    }

}



/*--- formatting done in "Sun Java Convention" style on 01-16-2000 ---*/

